#ifndef PluginH
#define PluginH
//---------------------------------------------------------------------------

#include "PluginInterface.h"
#include <System.hpp>
#include <Forms.hpp>
#include <vector>
#include <set>

/** \brief Info about dll device library */
struct DllInfo
{
	AnsiString name;
	AnsiString description;
	AnsiString file_version;			///< file version (as reported by OS)
	struct S_DLL_INTERFACE ver;
};

/** \brief Encapsulates dll plugin
*/
class Plugin
{
private:
	enum E_LIB_STATUS
	{
		E_OK = 0,
		E_LOADFAILED,			///< failed to load library - library not found or other dll dependency not met
		E_NOTVALID,				///< does not look like a plugin library
		E_INCOMPATIBLE_VER		///< incompatible plugin interface version
	};

	/** \brief Verify if dll looks correctly and fill dllinfo for valid dll
		\return loading status
	*/
	static enum E_LIB_STATUS VerifyDll(AnsiString filename, struct DllInfo* const dllinfo);
	/** \brief Add dll to list
	*/
	static void AddDll(const struct DllInfo &dllinfo);


	AnsiString filename;	///< dll name (with full path)
	HINSTANCE hInstance;	///< dll instance ptr
	// dll function pointers
	pfGetInterfaceDescription dllGetInterfaceDescription;
	pfSetCallbacks dllSetCallbacks;
	pfShowSettings dllShowSettings;
	pfChangeConnectionState dllChangeConnectionState;
	pfOnRx dllOnRx;
	pfOnTx dllOnTx;

	// CALLBACKS DEFINITIONS
	typedef void (__closure *CallbackSend)(void*, int);

public:
	/** \brief Make list of valid dlls in supplied location
	*/
	static void EnumerateDlls(AnsiString dir);
	/** \brief Constructor
		\param asDllName	Name of dll used by object to communicate with device
	*/
	Plugin(AnsiString asDllName);
	~Plugin();
	int Load(void);			///< load dll (dll name was supplied to constructor)
	AnsiString GetFilename(void) {
    	return filename;
	}
    /** Info about found device interface libraries */
	static std::vector<DllInfo> dlls;
	CallbackSend callbackSend;

	static std::set<class Plugin*> instances;

	static void __stdcall OnSend(void *cookie, void *buffer, unsigned int buffersize);

	static void Unload(void);

    /** \brief Show device settings window.
     *
     *  Show device settings window. Miniscope doesn't manage any settigs related to input devices. Device dll
     *  is responsible for storing/restoring data and can supply it's own window for this settings.
     */
	void ShowSettings(void) {
		if (dllShowSettings)
			dllShowSettings(Application->Handle);
	}
	void ChangeConnectionState(int state) {
		if (dllChangeConnectionState)
			dllChangeConnectionState(state);
	}
	void OnRx(void *buffer, unsigned int buffersize) {
		if (dllOnRx)
			dllOnRx(buffer, buffersize);
	}
	void OnTx(void *buffer, unsigned int buffersize) {
		if (dllOnTx)
			dllOnTx(buffer, buffersize);
	}
};

#endif
